@@ -1,7 +1,7 @@
 ;;; deb-view.el: View Debian package archive files with tar-mode.
 
 ;; Author:  Rick Macdonald (rickm@vsl.com)
-;; Version: 1.7
+;; Version: 1.9
 ;; Latest version: http://www.cuug.ab.ca/~macdonal/deb-view.el
 ;; Latest version: http://www.cuug.ab.ca/~macdonal/deb-view.el.gz
 
@@ -25,11 +25,11 @@
 ;;; Commentary:
 
 ;; deb-view presents the contents of debian package archive files for
-;; viewing. The viewing is done with emacs tar-mode, with a few
-;; enhancements for viewing compressed files and HTML files, and
-;; formatted man pages.
-;; The normal editing and saving features of tar-mode are not supported
-;; by deb-view.
+;; viewing. The viewing is done with the major mode "debview", which
+;; is derived from emacs tar-mode with a few enhancements for viewing
+;; compressed files, HTML files and formatted man pages.  The normal
+;; editing and saving features of tar-mode are not supported by
+;; deb-view.
 
 ;; deb-view includes a command called deb-find which requires that you
 ;; have the debian distribution directories on a local or mounted
@@ -40,8 +40,9 @@
 ;; external command. The find method passes your search specification to
 ;; egrep, whereas the locate method uses your string directly.
 
-;; deb-view extracts the control.tar.gz and data.tar.gz files from debian
-;; package and presents two buffers in tar-mode. See tar-mode for info.
+;; deb-view extracts the control.tar.gz and data.tar.gz files from
+;; debian package and presents two buffers in a derivitive of
+;; tar-mode. See tar-mode for info.
 
 ;; Required programs: ar, gzip.
 ;; Optionally required programs: nroff for formatting man pages.
@@ -86,12 +87,12 @@
 ;; in your `load-path'), `byte-compile-file' it, and put the following six
 ;; lines (with semi-colons removed) in your ~/.emacs file (or create ~/.emacs
 ;; if you don't have one):
-;; (autoload 'deb-find            "deb-view" "Dired Debian Archive File Finder" t)
-;; (autoload 'deb-view-mode       "deb-view" "Debian Archive File Mode" t)
-;; (autoload 'deb-view            "deb-view" "Dired Debian Archive File Viewer" t)
-;; (autoload 'deb-view-dired-view "deb-view" "Dired Debian Archive File Viewer" t)
-;; (setq auto-mode-alist (append '(("\\.deb$" . deb-view-mode)) auto-mode-alist))
-;; (define-key dired-mode-map     "\C-d"     'deb-view-dired-view) 
+;;(autoload 'deb-find            "deb-view" "Debian Archive File Finder" t)
+;;(autoload 'deb-view-mode       "deb-view" "Debian Archive File Mode" t)
+;;(autoload 'deb-view            "deb-view" "Debian Archive File Viewer" t)
+;;(autoload 'deb-view-dired-view "deb-view" "Debian Archive File Viewer" t)
+;;(setq auto-mode-alist (append '(("\\.deb$" . deb-view-mode)) auto-mode-alist))
+;;(define-key dired-mode-map     "\C-d"     'deb-view-dired-view) 
 
 ;; If you're not very familiar with emacs customization, here is a simpler
 ;; approach. Add this line to your ~/.emacs file (or create ~/.emacs if you
@@ -102,10 +103,12 @@
 ;; then just add the following to your ~./emacs file:
 ;;     (require 'deb-view)
 
-;; deb-view tries to be unobtrusive, but does the following:
-;; binds ^d in dired to deb-view-dired-view. (This is somewhat obsolete).
-;; binds q, N, W, and re-binds v in tar-mode. These functions are then
-;;       available in standard tar-mode on real tar files.
+;; deb-view is mostly unobtrusive, but does bind ^d in dired to
+;; deb-view-dired-view.  The "debview" mode is derived from
+;; tar-mode.el using derived.el. Compared to tar-mode, debview-mode
+;; binds q, N, W, and re-binds v. Also, the normal editing and saving
+;; features of tar-mode are not supported by debview mode and those
+;; keys are disabled.
 
 ;; 3) Configuration
 
@@ -119,15 +122,15 @@
 ;; uses egrep and therefore understands complex regular expressions.
 ;; You might want to bind deb-find to a special key. I use ^C^D like this:
 ;; (define-key ctl-x-map "\C-d" 'deb-find)
-;; Note that this key is normally the brief list-directory command, but I
-;; never used it anyway.
+;; Note that this key is normally the brief list-directory command, a
+;; command that I never used anyway.
 
 
 ;;; Usage:
 
 ;; In dired, press f or e on the dired line of the .deb file to view.
-;; You can also use ^d, but this is somewhat obsolete now and might
-;; be removed in the future.
+;; You can also use ^d, which is actually slightly faster since the
+;; deb file isn't loaded into a buffer needlessly.
 
 ;; Or, execute: ESCAPE x deb-view RETURN, and enter the .deb file name
 ;; at the prompt.
@@ -137,7 +140,7 @@
 ;; Launch deb-view by selecting a deb file with the middle mouse button,
 ;; or RETURN or ^c^c. Exit this buffer with "q".
 
-;; You are shown two tar files in tar-mode (see tar-mode for help).
+;; You are shown two tar files in debview-mode (see tar-mode for help).
 ;; In the case of old .deb format files, the control info is shown
 ;; but not the other files of control.tar, such as install scripts.
 ;; Note that regular tar-mode commands e, f and RETURN show raw files
@@ -150,12 +153,10 @@
 ;; N - Like in dired, formats man pages for viewing, with the
 ;;     additional smarts to uncompress .gz and .Z man files for viewing.
 ;; W - use w3-mode to view an HTML file.
-;; These functions are also available in tar-mode on real tar files
-;; when deb-view is loaded.
 
 ;; To view files not supported by deb-view, such as graphics, use the
-;; copy command in tar-mode ("c") to copy the file to a temp directory.
-;; You can then do what you want to the file.
+;; copy command ("c") to copy the file to a temp directory.  You can
+;; then do what you want to the file.
 
 
 ;;; Changelog:
@@ -185,6 +186,13 @@
 ;;        it directly. It was breaking actual compilation buffer keymaps,
 ;;        such as grep mode.
 
+;; 1.8  - fixed deb-find when deb-find-method is set to "find". It wasn't
+;;        adding "/*" to the end of the directory name for the find command.
+
+;; 1.9  - Added support for handling remote deb files (ange-ftp).
+;;      - reworked to use derived.el instead of messing with tar-mode
+;;        directly. (Thanks to era eriksson <era@iki.fi>)
+
 
 ;; User variables:
 
@@ -192,9 +200,9 @@
   "*Program to use for uncompression of .gz and .Z files in deb-view.")
 
 ;; Note the following useful variable from tar-mode:
-;; (defvar tar-mode-show-date nil
-;;  "*Non-nil means Tar mode should show the date/time of each subfile.
-;; This information is useful, but it takes screen space away from file names.")
+;;(defvar tar-mode-show-date nil
+;; "*Non-nil means Tar mode should show the date/time of each subfile.
+;;This information is useful, but it takes screen space away from file names.")
 
 (defvar deb-find-method "find"
   "Internal deb-find methods supported: locate or find.
@@ -206,10 +214,43 @@ See also the variable deb-find-directory.")
 
 ;;; Code:
 
+(define-derived-mode debview-mode tar-mode "debview"
+  "Major mode for debview.\n\n\\{debview-mode-map}")
+
+;; Prohibit things that tar-mode does that deb-view doesn't:
+(define-key debview-mode-map "\C-d" 'undefined)
+(define-key debview-mode-map "G"    'undefined)
+(define-key debview-mode-map "M"    'undefined)
+(define-key debview-mode-map "O"    'undefined)
+(define-key debview-mode-map "d"    'undefined)
+(define-key debview-mode-map "g"    'undefined)
+(define-key debview-mode-map "r"    'undefined)
+(define-key debview-mode-map "u"    'undefined)
+(define-key debview-mode-map "x"    'undefined)
+(define-key debview-mode-map ""   'undefined)
+             
+(define-key debview-mode-map "?"    'deb-view-help)
+(define-key debview-mode-map "q"    'deb-view-dired-view-cleanup)
+(define-key debview-mode-map "N"    'deb-view-tar-man)
+(define-key debview-mode-map "W"    'deb-view-tar-w3)
+(define-key debview-mode-map "v"    'deb-view-tar-view)
+(define-key debview-mode-map [up]   'tar-previous-line)
+(define-key debview-mode-map [down] 'tar-next-line)
+(define-key debview-mode-map "\eOA" 'tar-previous-line)
+(define-key debview-mode-map "\eOB" 'tar-next-line)
+(define-key debview-mode-map "\e[A" 'tar-previous-line)
+(define-key debview-mode-map "\e[B" 'tar-next-line)
+
 (defvar deb-view-dired-view-return-buffer ""
   "Return to this buffer after deb-view-dired-view-cleanup is called.")
 (make-variable-buffer-local 'deb-view-dired-view-return-buffer)
 
+(defvar deb-view-tempfile ""
+  "Flag saying if the deb file is temporary (ange-ftp) and needs deleting.")
+
+(defvar deb-view-file-name ""
+  "The file name being processed by deb-view.")
+
 ;; You might not like the key bindings that I chose:
 (if (featurep 'dired)
     (define-key dired-mode-map "\C-d" 'deb-view-dired-view)
@@ -218,55 +259,45 @@ See also the variable deb-find-directory.")
    (function (lambda ()
                (define-key dired-mode-map "\C-d" 'deb-view-dired-view)))))
 
-;; You might not like the key bindings that I chose:
-(add-hook
- 'tar-mode-hook
- (function (lambda ()
-             (define-key tar-mode-map "\C-d"   'undefined)
-             (define-key tar-mode-map "G"      'undefined)
-             (define-key tar-mode-map "M"      'undefined)
-             (define-key tar-mode-map "O"      'undefined)
-             (define-key tar-mode-map "d"      'undefined)
-             (define-key tar-mode-map "g"      'undefined)
-             (define-key tar-mode-map "r"      'undefined)
-             (define-key tar-mode-map "u"      'undefined)
-             (define-key tar-mode-map "x"      'undefined)
-             (define-key tar-mode-map ""     'undefined)
-             
-             (define-key tar-mode-map "?"    'deb-view-help)
-             (define-key tar-mode-map "q"    'deb-view-dired-view-cleanup)
-             (define-key tar-mode-map "N"    'deb-view-tar-man)
-             (define-key tar-mode-map "W"    'deb-view-tar-w3)
-             (define-key tar-mode-map "v"    'deb-view-tar-view)
-             (define-key tar-mode-map [up]   'tar-previous-line)
-             (define-key tar-mode-map [down] 'tar-next-line)
-             (define-key tar-mode-map "\eOA" 'tar-previous-line)
-             (define-key tar-mode-map "\eOB" 'tar-next-line)
-             (define-key tar-mode-map "\e[A" 'tar-previous-line)
-             (define-key tar-mode-map "\e[B" 'tar-next-line))))
-
 (defun deb-view-dired-view ()
-  "View the control and data files of the Debian package archive file on this line.
-Press \"q\" in either window to kill both buffers and return to the dired buffer. See deb-view."
+  "View Debian package control and data files.
+Press \"q\" in either window to kill both buffers
+and return to the dired buffer. See deb-view."
   (interactive)
   (deb-view (dired-get-filename)))
 
 (defun deb-view (debfile)
-  "View the control and data files of a Debian package archive file. Press \"q\"
-in either window to kill both buffers.
+  "View Debian package control and data files.
+Press \"q\" in either window to kill both buffers.
 
 In dired, press ^d on the dired line of the .deb file to view.
 Or, execute: ESCAPE x deb-view RETURN, and enter the .deb file name
 at the prompt."
   (interactive "fdeb file to view: ")
   (if (and (or (string-match "Lucid" emacs-version)
-	       (string-match "XEmacs" emacs-version))
-	   (>= emacs-major-version 21))
+ 	       (string-match "XEmacs" emacs-version))
+ 	   (>= emacs-major-version 21))
       (require 'view-less)
     (require 'view))
   (require 'view)
-  (let* ((info-buffer-name (concat (file-name-nondirectory debfile) "-INFO"))
-         (data-buffer-name (concat (file-name-nondirectory debfile) "-DATA"))
+  (if (< (nth 1 (file-attributes debfile)) 0)
+      (progn
+        ;; This is a remote file.
+        ;; Call view-file to force ange-ftp to get it first.
+        (message "deb-view remote file: %s" debfile)
+        (find-file debfile))
+    ;; This is a local file.
+    (setq debfile (expand-file-name debfile))
+    ;;(message "deb-view local file: %s" debfile)
+    (setq deb-view-file-name debfile)
+    (setq deb-view-tempfile nil)
+    (deb-view-process debfile)))
+
+(defun deb-view-process (debfile)
+  "View Debian Archive Files."
+  (let* ((deb-view-buffer-name (file-name-nondirectory deb-view-file-name))
+         (info-buffer-name (concat deb-view-buffer-name "-INFO"))
+         (data-buffer-name (concat deb-view-buffer-name "-DATA"))
          (info-buffer (progn (and (get-buffer info-buffer-name)
                                   (kill-buffer (get-buffer info-buffer-name)))
                              (get-buffer-create info-buffer-name)))
@@ -277,6 +308,7 @@ at the prompt."
 	 (coding-system-for-read 'no-conversion)
 	 file-buffer
          new-archive-format)
+    (message "deb-view processing deb file %s..." deb-view-buffer-name)
     ;; info
     (setq file-buffer (get-buffer-create " *file-data*"))
     (setq new-archive-format
@@ -301,13 +333,21 @@ at the prompt."
                         (concat "ar -p " debfile
                                 " control.tar.gz | gzip -cd"))
           (goto-char 1)
-          (setq buffer-file-name (concat debfile "-INFO"))
+          (setq buffer-file-name (concat deb-view-file-name "-INFO"))
           (if (fboundp 'set-buffer-multibyte) (set-buffer-multibyte nil))
-          (tar-mode)
+          (debview-mode)
+          ;; Turn off view-mode in this buffer:
+          (make-variable-buffer-local 'view-mode-hook)
+          (add-hook
+           'view-mode-hook
+           (function (lambda ()
+                       (view-mode -1)
+                       (setq view-exit-action 'deb-view-dired-view-cleanup))))
+          (message "deb-view processing deb file %s..." deb-view-buffer-name)
           (tar-next-line 1)
           (switch-to-buffer info-buffer t))
       ;; Old deb format
-      (message "old dpkg binary format")
+      (message "deb-view old dpkg binary format")
       (call-process shell-file-name nil t nil shell-command-switch
                     (concat "dpkg-deb -I " debfile))
       (setq buffer-read-only t)
@@ -326,33 +366,42 @@ at the prompt."
                       (concat "ar -p " debfile " data.tar.gz | gzip -cd")
                     (concat "dpkg-deb --fsys-tarfile " debfile)))
     (goto-char 1)
-    (setq buffer-file-name (concat debfile "-DATA"))
+    (setq buffer-file-name (concat deb-view-file-name "-DATA"))
     (if (fboundp 'set-buffer-multibyte) (set-buffer-multibyte nil))
-    (tar-mode)
+    (debview-mode)
+    (message "deb-view processing deb file %s..." deb-view-buffer-name)
     (tar-next-line 1)
     (setq deb-view-dired-view-return-buffer return-buffer)
     (set-buffer-modified-p nil)
     (setq buffer-read-only t)
     (switch-to-buffer-other-window data-buffer)
     (if new-archive-format (other-window 1))
-    (message "? for help. q to quit.")))
+    (if deb-view-tempfile
+        (progn
+        (message "deb-view deleting tempfile: %s" debfile)
+        (delete-file debfile)))
+    (message "deb-view: ? for help. q to quit.")))
 
 (defun deb-view-mode ()
   "View mode for Debian Archive Files."
   (interactive)
-  (let ((deb-file buffer-file-name)
+  (let ((debfile buffer-file-name)
         (return-buffer (nth 0 (buffer-list)))
         (curbuf (current-buffer)))
+    (setq deb-view-file-name debfile)
+    (if (< (nth 1 (file-attributes debfile)) 0)
+        (progn
+          (message "deb-view remote file: %s" debfile)
+          (setq debfile (make-temp-name "/tmp/deb-view."))
+          ;;(message "deb-view processing deb file %s..." debfile)
+          (write-file debfile)
+          (setq deb-view-tempfile t))
+      ;;(message "deb-view local file: %s" debfile)
+      (setq deb-view-tempfile nil))
     (set-buffer return-buffer)
     (kill-buffer curbuf)
-    ;; If you ever have a problem (with an emacs version newer than 19.34)
-    ;; with the "unread-command-events" method, use the deb-view call
-    ;; directly.
-    ;; (deb-view buffer)
-    (setq unread-command-events
-          (append "\exdeb-view" "\C-m" deb-file "\C-m"
-                  unread-command-events))))
-  
+    (deb-view-process debfile)))
+
 (defun deb-find ()
   "Search for deb files using the method specified by the variable
 deb-find-method, and collect output in a buffer.
@@ -366,18 +415,21 @@ easily repeat a deb-find command."
                                          nil nil nil 'deb-find-history))
          (output-buffer-name "*deb-find*")
          (command (cond ((string-equal deb-find-method "locate")
-                         (concat "locate '" deb-file-string "' | egrep '\.deb$'"))
+                         (concat "locate '" deb-file-string
+                                 "' | egrep '\.deb$'"))
                         ((string-equal deb-find-method "find")
-                         (concat "find " deb-find-directory " | egrep '"
+                         (concat "find " deb-find-directory "/* | egrep '"
                                  deb-file-string "' | egrep '\.deb$'"))
                         (t
                          (concat deb-find-method " '" deb-file-string "'")))))
     (compile-internal command "Not applicable in deb-find" "deb-find" nil nil
                       (function (lambda (mode) output-buffer-name)))
     (switch-to-buffer-other-window output-buffer-name)
-    (setq deb-view-find-minor-mode-map (copy-keymap compilation-minor-mode-map))
+    (setq deb-view-find-minor-mode-map
+          (copy-keymap compilation-minor-mode-map))
     (use-local-map deb-view-find-minor-mode-map)
-    (define-key deb-view-find-minor-mode-map [mouse-2]  'deb-find-mouse-deb-view)
+    (define-key deb-view-find-minor-mode-map [mouse-2]
+      'deb-find-mouse-deb-view)
     (define-key deb-view-find-minor-mode-map "\C-c\C-c" 'deb-find-deb-view)
     (define-key deb-view-find-minor-mode-map "\C-m"     'deb-find-deb-view)
     (define-key deb-view-find-minor-mode-map "?"        'deb-find-help)
@@ -387,12 +439,12 @@ easily repeat a deb-find command."
     (define-key deb-view-find-minor-mode-map "\M-{" 'undefined)
     (define-key deb-view-find-minor-mode-map "\M-}" 'undefined)
     (beginning-of-buffer)
-    (message "? for help. q to quit.")))
+    (message "deb-view: ? for help. q to quit.")))
 
 
 ;;; Internal functions:
 
-(defvar deb-view-version "1.7"
+(defvar deb-view-version "1.9"
   "The version of deb-view.")
 
 (defun deb-view-version ()
@@ -446,7 +498,7 @@ q - quit deb-find.")
   (with-output-to-temp-buffer "*Help*"
     (princ (format "deb-view mode:    version %s" (deb-view-version)))
     (princ "
-Based-on tar-mode, with additional features for viewing deb files.
+Derived from tar-mode, with additional features for viewing deb files.
 Execute \"^h m\" to see tar-mode bindings.
 
 You are shown two tar files in tar-mode (see tar-mode for help).
